"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.removeUnwantedCharacters = exports.processMarkup = exports.createSlateMarkup = exports.createElementFromHTML = void 0;

var _escape = _interopRequireDefault(require("lodash/escape"));

// do not remove \t from \times, \triangle, \tan, \theta or \therefore
var tSymbols = 'imes|riangle|an|heta|herefore'; // do not remove \n from \nthroot, \nparallel, \ncong, \napprox, \neq, \ne or \nsim

var nSymbols = 'throot|parallel|cong|approx|eq|e|sim'; // match all \t and \n that are not part of math symbols that starts with \t or \n

var matchTabAndNewLine = new RegExp("(\\t(?!".concat(tSymbols, "))|(\\n(?!").concat(nSymbols, "))|(\\\\t(?!").concat(tSymbols, "))|(\\\\n(?!").concat(nSymbols, "))"), 'g');

var removeUnwantedCharacters = function removeUnwantedCharacters(markup) {
  return markup.replace(matchTabAndNewLine, '').replace(/\\"/g, '"').replace(/\\\//g, '/');
};

exports.removeUnwantedCharacters = removeUnwantedCharacters;

var createElementFromHTML = function createElementFromHTML(htmlString) {
  var div = document.createElement('div');
  div.innerHTML = htmlString.trim();
  return div;
};

exports.createElementFromHTML = createElementFromHTML;

var processMarkup = function processMarkup(markup) {
  var newMarkup = removeUnwantedCharacters(markup);
  var slateMarkup = createElementFromHTML(newMarkup);
  slateMarkup.querySelectorAll('[data-type="inline_dropdown"]').forEach(function (s) {
    s.replaceWith("{{".concat(s.dataset.index, "}}"));
  });
  return slateMarkup.innerHTML;
};

exports.processMarkup = processMarkup;
var REGEX = /\{\{(\d+)\}\}/g;

var createSlateMarkup = function createSlateMarkup(markup, choices) {
  var newMarkup = removeUnwantedCharacters(markup);

  var createSelect = function createSelect(index) {
    var correctChoice = choices[index] && choices[index].find(function (c) {
      return c.correct;
    });

    if (!correctChoice || !correctChoice.value) {
      correctChoice = {
        id: '',
        value: '',
        label: ''
      };
    }

    return "<span data-type=\"inline_dropdown\" data-index=\"".concat(index, "\" data-value=\"").concat((0, _escape["default"])(correctChoice.label), "\"></span>");
  };

  return newMarkup.replace(REGEX, function (match, g) {
    return createSelect(g);
  });
};

exports.createSlateMarkup = createSlateMarkup;
//# sourceMappingURL=markupUtils.js.map