"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getScore = exports.createCorrectResponseSession = void 0;
exports.model = model;
exports.normalize = void 0;
exports.outcome = outcome;
exports.validate = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _map = _interopRequireDefault(require("lodash/map"));

var _reduce = _interopRequireDefault(require("lodash/reduce"));

var _isEmpty = _interopRequireDefault(require("lodash/isEmpty"));

var _controllerUtils = require("@pie-lib/pie-toolbox/controller-utils");

var _defaults = _interopRequireDefault(require("./defaults"));

var _utils = require("./utils");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

var getFeedback = function getFeedback(correct) {
  if (correct) {
    return 'correct';
  }

  return 'incorrect';
};

var normalize = function normalize(question) {
  return _objectSpread(_objectSpread({}, _defaults["default"]), question);
};
/**
 *
 * @param {*} question
 * @param {*} session
 * @param {*} env
 * @param {*} updateSession - optional - a function that will set the properties passed into it on the session.
 */


exports.normalize = normalize;

function model(question, session, env, updateSession) {
  return new Promise( /*#__PURE__*/function () {
    var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(resolve) {
      var normalizedQuestion, _ref2, _ref2$value, value, choices, feedback, allCorrectResponses, respAreaLength, correctResponses, _loop, i, _ret, lockChoiceOrder, keys, _i, key, teacherInstructions, rationale, choicesWillNullRationales, out;

      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              normalizedQuestion = normalize(question);
              _ref2 = session || {}, _ref2$value = _ref2.value, value = _ref2$value === void 0 ? {} : _ref2$value;
              choices = (0, _reduce["default"])(normalizedQuestion.choices, function (obj, area, key) {
                obj[key] = (0, _map["default"])(area, function (choice) {
                  return choice;
                });
                return obj;
              }, {});
              feedback = {};

              if (!(env.mode === 'evaluate')) {
                _context.next = 17;
                break;
              }

              allCorrectResponses = (0, _utils.getAllCorrectResponses)(normalizedQuestion);
              respAreaLength = Object.keys(allCorrectResponses).length;
              correctResponses = 0;

              _loop = function _loop(i) {
                var result = (0, _reduce["default"])(allCorrectResponses, function (obj, choices, key) {
                  var answer = value && value[key] || '';
                  var correctChoice = choices[i] || '';
                  var isCorrect = answer && correctChoice && correctChoice === answer;
                  obj.feedback[key] = getFeedback(isCorrect);

                  if (isCorrect) {
                    obj.correctResponses += 1;
                  }

                  return obj;
                }, {
                  correctResponses: 0,
                  feedback: {}
                });

                if (result.correctResponses >= correctResponses) {
                  correctResponses = result.correctResponses;
                  feedback = result.feedback;
                }

                if (result.correctResponses === respAreaLength) {
                  return "break";
                }
              };

              i = 0;

            case 10:
              if (!(i < respAreaLength)) {
                _context.next = 17;
                break;
              }

              _ret = _loop(i);

              if (!(_ret === "break")) {
                _context.next = 14;
                break;
              }

              return _context.abrupt("break", 17);

            case 14:
              i++;
              _context.next = 10;
              break;

            case 17:
              lockChoiceOrder = (0, _controllerUtils.lockChoices)(normalizedQuestion, session, env);

              if (lockChoiceOrder) {
                _context.next = 29;
                break;
              }

              keys = Object.keys(choices);
              _i = 0;

            case 21:
              if (!(_i < keys.length)) {
                _context.next = 29;
                break;
              }

              key = keys[_i];
              _context.next = 25;
              return (0, _controllerUtils.getShuffledChoices)(choices[key], session, updateSession, 'value');

            case 25:
              choices[key] = _context.sent;

            case 26:
              _i++;
              _context.next = 21;
              break;

            case 29:
              teacherInstructions = null;
              rationale = null;
              choicesWillNullRationales = (Object.keys(choices) || []).reduce(function (acc, currentValue) {
                acc[currentValue] = (choices[currentValue] || []).map(function (choice) {
                  return _objectSpread(_objectSpread({}, choice), {}, {
                    rationale: null
                  });
                });
                return acc;
              }, {});

              if (env.role === 'instructor' && (env.mode === 'view' || env.mode === 'evaluate')) {
                rationale = normalizedQuestion.rationaleEnabled ? normalizedQuestion.rationale : null;
                teacherInstructions = normalizedQuestion.teacherInstructionsEnabled ? normalizedQuestion.teacherInstructions : null;
                choices = normalizedQuestion.choiceRationaleEnabled ? normalizedQuestion.choices : choicesWillNullRationales;
              } else {
                rationale = null;
                teacherInstructions = null;
                choices = choicesWillNullRationales;
              }

              out = {
                disabled: env.mode !== 'gather',
                mode: env.mode,
                prompt: normalizedQuestion.promptEnabled ? normalizedQuestion.prompt : null,
                displayType: normalizedQuestion.displayType,
                markup: normalizedQuestion.markup,
                choices: choices,
                feedback: feedback,
                responseCorrect: env.mode === 'evaluate' ? getScore(normalizedQuestion, session) === 1 : undefined,
                rationale: rationale,
                teacherInstructions: teacherInstructions,
                language: normalizedQuestion.language,
                extraCSSRules: normalizedQuestion.extraCSSRules
              };
              resolve(out);

            case 35:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }());
}

var getScore = function getScore(config, session) {
  var _ref3 = session || {},
      _ref3$value = _ref3.value,
      value = _ref3$value === void 0 ? {} : _ref3$value;

  var maxScore = config && config.choices ? Object.keys(config.choices).length : 0;
  var allCorrectResponses = (0, _utils.getAllCorrectResponses)(config);
  var correctCount = 0;

  var _loop2 = function _loop2(i) {
    var result = (0, _reduce["default"])(allCorrectResponses, function (total, choices, key) {
      var answer = value && value[key] || '';
      var correctChoice = choices[i] || '';

      if (correctChoice && answer && correctChoice === answer) {
        return total;
      }

      return total - 1;
    }, maxScore);

    if (result > correctCount) {
      correctCount = result;
    }

    if (result === maxScore) {
      return "break";
    }
  };

  for (var i = 0; i < maxScore; i++) {
    var _ret2 = _loop2(i);

    if (_ret2 === "break") break;
  }

  var str = (correctCount / maxScore).toFixed(2);
  return parseFloat(str);
};
/**
 *
 * The score is partial by default for checkbox mode, allOrNothing for radio mode.
 * To disable partial scoring for checkbox mode you either set model.partialScoring = false or env.partialScoring =
 * false. the value in `env` will override the value in `model`.
 * @param {Object} model - the main model
 * @param {boolean} model.partialScoring - is partial scoring enabled (if undefined set to to true)
 * @param {*} session
 * @param {Object} env
 * @param {boolean} env.partialScoring - is partial scoring enabled (if undefined default to true) This overrides
 *   `model.partialScoring`.
 */


exports.getScore = getScore;

function outcome(model, session) {
  var env = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return new Promise(function (resolve) {
    if (!session || (0, _isEmpty["default"])(session)) {
      resolve({
        score: 0,
        empty: true
      });
    }

    var partialScoringEnabled = _controllerUtils.partialScoring.enabled(model, env);

    var score = getScore(model, session);
    resolve({
      score: partialScoringEnabled ? score : Math.floor(score),
      empty: false
    });
  });
}

var createCorrectResponseSession = function createCorrectResponseSession(question, env) {
  return new Promise(function (resolve) {
    if (env.mode !== 'evaluate' && env.role === 'instructor') {
      var choices = question.choices;
      var _value = {};

      if (choices) {
        Object.keys(choices).forEach(function (key, i) {
          var correctChoices = choices[key] && choices[key].filter(function (c) {
            return c.correct;
          });
          _value[i] = correctChoices && correctChoices[0].value;
        });
      }

      resolve({
        id: '1',
        value: _value
      });
    } else {
      resolve(null);
    }
  });
}; // remove all html tags


exports.createCorrectResponseSession = createCorrectResponseSession;

var getInnerText = function getInnerText(html) {
  return (html || '').replaceAll(/<[^>]*>/g, '');
}; // remove all html tags except img, iframe and source tag for audio


var getContent = function getContent(html) {
  return (html || '').replace(/(<(?!img|iframe|source)([^>]+)>)/gi, '');
};

var validate = function validate() {
  var model = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var markup = model.markup,
      choices = model.choices;
  var maxResponseAreas = config.maxResponseAreas,
      maxResponseAreaChoices = config.maxResponseAreaChoices;
  var errors = {};
  ['teacherInstructions', 'prompt', 'rationale'].forEach(function (field) {
    var _config$field;

    if ((_config$field = config[field]) !== null && _config$field !== void 0 && _config$field.required && !getContent(model[field])) {
      errors[field] = 'This field is required.';
    }
  });
  var nbOfResponseAreas = ((markup || '').match(/\{\{(\d+)\}\}/g) || []).length;

  if (nbOfResponseAreas > maxResponseAreas) {
    errors.responseAreasError = "No more than ".concat(maxResponseAreas, " response areas should be defined.");
  } else if (nbOfResponseAreas < 1) {
    errors.responseAreasError = 'There should be defined at least 1 response area.';
  }

  (Object.keys(choices) || []).forEach(function (choiceKey) {
    if (choices[choiceKey] && choices[choiceKey].length > maxResponseAreaChoices) {
      errors.responseAreaChoicesError = "No more than ".concat(maxResponseAreaChoices, " choices per response area should be defined.");
    }
  });
  return errors;
};

exports.validate = validate;
//# sourceMappingURL=index.js.map